@@ -36,8 +36,12 @@ static inline unsigned long xnarch_get_sched_latency (void)
 	return 8500;
 #elif defined(CONFIG_ARCH_AT91SAM9263)
 	return 11000;
-#elif defined(CONFIG_ARCH_OMAP3)
+#elif defined(CONFIG_ARCH_MX51)
 	return 5000;
+#elif defined(CONFIG_ARCH_MX53)
+	return 5000;
+#elif defined(CONFIG_ARCH_OMAP)
+	return cpu_is_omap44xx() ? 2500 : 5000;
 #else
 	return 9500;	/* XXX sane ? */
 #endif
