@@ -3,7 +3,7 @@
  *
  * 64-bit PowerPC adoption
  *   copyright (C) 2005 Taneli Vähäkangas and Heikki Lindholm
- *   
+ *
  * Xenomai is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -35,9 +35,4 @@ static inline void xnarch_relay_tick(void)
 	rthal_irq_host_pend(RTHAL_TIMER_IRQ);
 }
 
-static inline void xnarch_announce_tick(void)
-{
-	/* empty */
-}
-
 #endif /* !_XENO_ASM_POWERPC_BITS_INTR_H */
