@@ -1,4 +1,4 @@
-/* 
+/*
  * @note Copyright (C) 2001,2002,2003 Philippe Gerum <rpm@xenomai.org>.
  *
  * Xenomai is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 
 #include <nucleus/types.h>
 
-/*! @ingroup nucleus 
+/*! @ingroup nucleus
   @defgroup nucleus_state_flags Thread state flags.
   @brief Bits reporting permanent or transient states of thread.
   @{
@@ -62,7 +62,7 @@
 #define XNFPU     0x00100000 /**< Thread uses FPU */
 #define XNSHADOW  0x00200000 /**< Shadow thread */
 #define XNROOT    0x00400000 /**< Root thread (that is, Linux/IDLE) */
-#define XNSWREP   0x00800000 /**< Mode switch already reported */
+#define XNOTHER   0x00800000 /**< Non real-time shadow (prio=0) */
 
 /*! @} */ /* Ends doxygen comment group: nucleus_state_flags */
 
@@ -97,7 +97,7 @@
 #define XNTHREAD_STATE_SPARE3  0x80000000
 #define XNTHREAD_STATE_SPARES  0xf0000000
 
-/*! @ingroup nucleus 
+/*! @ingroup nucleus
   @defgroup nucleus_info_flags Thread information flags.
   @brief Bits reporting events notified to the thread.
   @{
@@ -117,6 +117,7 @@
 #define XNABORT   0x00000200 /**< Thread is being aborted */
 #define XNCANPND  0x00000400 /**< Cancellation request is pending */
 #define XNAMOK    0x00000800 /**< Runaway, watchdog signal pending (shadow only) */
+#define XNSWREP   0x00001000 /**< Mode switch already reported */
 
 /* These information flags are available to the real-time interfaces */
 #define XNTHREAD_INFO_SPARE0  0x10000000
@@ -234,7 +235,7 @@ typedef struct xnthread {
 	int cprio;			/* Current priority */
 
 	u_long schedlck;		/*!< Scheduler lock count. */
-	
+
 	xnpholder_t rlink;		/* Thread holder in ready queue */
 
 	xnpholder_t plink;		/* Thread holder in synchronization queue(s) */
@@ -254,11 +255,13 @@ typedef struct xnthread {
 	struct xnsynch *wchan;		/* Resource the thread pends on */
 
 	struct xnsynch *wwake;		/* Wait channel the thread was resumed from */
-	
+
+	int hrescnt;			/* Held resources count */
+
 	xntimer_t rtimer;		/* Resource timer */
 
 	xntimer_t ptimer;		/* Periodic timer */
-	
+
 	xnsigmask_t signals;		/* Pending core signals */
 
 	xnticks_t rrperiod;		/* Allotted round-robin period (ticks) */
@@ -326,9 +329,7 @@ typedef struct xnthread {
 	void *cookie;		/* Cookie to pass to the entry routine */
 
 #ifdef CONFIG_XENO_OPT_PERVASIVE
-	unsigned long __user *u_mode;	/* Thread mode variable in userland. */
-
-	unsigned u_sigpending;		/* One bit per skin */
+	unsigned long *u_mode;	/* Thread mode variable shared with userland. */
 #endif /* CONFIG_XENO_OPT_PERVASIVE */
 
     XNARCH_DECL_DISPLAY_CONTEXT();
@@ -389,10 +390,13 @@ typedef struct xnhook {
 #define xnthread_get_exectime(thread)      xnstat_exectime_get_total(&(thread)->stat.account)
 #define xnthread_get_lastswitch(thread)    xnstat_exectime_get_last_switch((thread)->sched)
 #ifdef CONFIG_XENO_OPT_PERVASIVE
-#define xnthread_sigpending(thread) ((thread)->u_sigpending)
-#define xnthread_set_sigpending(thread, pending) \
-	((thread)->u_sigpending = (pending))
-#endif /* CONFIG_XENO_OPT_PERVASIVE */
+#define xnthread_inc_rescnt(thread)        ({ (thread)->hrescnt++; })
+#define xnthread_dec_rescnt(thread)        ({ --(thread)->hrescnt; })
+#define xnthread_get_rescnt(thread)        ((thread)->hrescnt)
+#else /* !CONFIG_XENO_OPT_PERVASIVE */
+#define xnthread_inc_rescnt(thread)        do { } while (0)
+#define xnthread_dec_rescnt(thread)        do { } while (0)
+#endif /* !CONFIG_XENO_OPT_PERVASIVE */
 #ifdef CONFIG_XENO_OPT_WATCHDOG
 #define xnthread_amok_p(thread)            xnthread_test_info(thread, XNAMOK)
 #define xnthread_clear_amok(thread)        xnthread_clear_info(thread, XNAMOK)
