@@ -42,12 +42,7 @@
 #endif
 #include <asm/system.h>
 #include <asm/irq.h>
-#include <asm/uaccess.h>
-#include <asm/unistd.h>
 #include <asm/xenomai/hal.h>
-#ifdef CONFIG_PROC_FS
-#include <linux/proc_fs.h>
-#endif /* CONFIG_PROC_FS */
 #include <stdarg.h>
 
 MODULE_LICENSE("GPL");
@@ -66,22 +61,11 @@ static unsigned long supported_cpus_arg = -1;
 module_param_named(supported_cpus, supported_cpus_arg, ulong, 0444);
 
 cpumask_t rthal_supported_cpus;
-EXPORT_SYMBOL(rthal_supported_cpus);
+EXPORT_SYMBOL_GPL(rthal_supported_cpus);
 #endif /* CONFIG_SMP */
 
-static struct {
-
-    void (*handler) (void *cookie);
-    void *cookie;
-    const char *name;
-    unsigned long hits[RTHAL_NR_CPUS];
-
-} rthal_apc_table[RTHAL_NR_APCS];
-
 static int rthal_init_done;
 
-static unsigned long rthal_apc_map;
-
 static rthal_spinlock_t rthal_apc_lock = RTHAL_SPIN_LOCK_UNLOCKED;
 
 static atomic_t rthal_sync_count = ATOMIC_INIT(1);
@@ -93,6 +77,13 @@ struct rthal_calibration_data rthal_tunables;
 rthal_trap_handler_t rthal_trap_handler;
 
 unsigned rthal_realtime_faults[RTHAL_NR_CPUS][RTHAL_NR_FAULTS];
+EXPORT_SYMBOL_GPL(rthal_realtime_faults);
+
+unsigned long rthal_apc_map;
+EXPORT_SYMBOL_GPL(rthal_apc_map);
+
+struct rthal_apc_desc rthal_apc_table[RTHAL_NR_APCS];
+EXPORT_SYMBOL_GPL(rthal_apc_table);
 
 volatile int rthal_sync_op;
 
@@ -303,67 +294,6 @@ int rthal_irq_release(unsigned irq)
  * - Xenomai domain context.
  */
 
-/**
- * @fn int rthal_irq_affinity (unsigned irq,cpumask_t cpumask,cpumask_t *oldmask)
- *
- * @brief Set/Get processor affinity for external interrupt.
- *
- * On SMP systems, this service ensures that the given interrupt is
- * preferably dispatched to the specified set of processors. The
- * previous affinity mask is returned by this service.
- *
- * @param irq The interrupt source whose processor affinity is
- * affected by the operation. Only external interrupts can have their
- * affinity changed/queried, thus virtual interrupt numbers allocated
- * by rthal_alloc_virq() are invalid values for this parameter.
- *
- * @param cpumask A list of CPU identifiers passed as a bitmask
- * representing the new affinity for this interrupt. A zero value
- * cause this service to return the current affinity mask without
- * changing it.
- *
- * @param oldmask If non-NULL, a pointer to a memory area which will
- * bve overwritten by the previous affinity mask used for this
- * interrupt source, or a zeroed mask if an error occurred.  This
- * service always returns a zeroed mask on uniprocessor systems.
- *
- * @return 0 is returned upon success. Otherwise:
- *
- * - -EINVAL is returned if @a irq is invalid.
- *
- * Environments:
- *
- * This service can be called from:
- *
- * - Linux domain context.
- */
-
-#ifdef CONFIG_SMP
-
-int rthal_irq_affinity(unsigned irq, cpumask_t cpumask, cpumask_t *oldmask)
-{
-    cpumask_t _oldmask;
-
-    if (irq >= IPIPE_NR_XIRQS)
-	return -EINVAL;
-
-    _oldmask = rthal_set_irq_affinity(irq, cpumask);
-
-    if (oldmask)
-	*oldmask = _oldmask;
-
-    return cpus_empty(_oldmask) ? -EINVAL : 0;
-}
-
-#else /* !CONFIG_SMP */
-
-int rthal_irq_affinity(unsigned irq, cpumask_t cpumask, cpumask_t *oldmask)
-{
-    return 0;
-}
-
-#endif /* CONFIG_SMP */
-
 /**
  * @fn int rthal_trap_catch (rthal_trap_handler_t handler)
  *
@@ -451,7 +381,7 @@ static int rthal_apc_thread(void *data)
     while (!kthread_should_stop()) {
 	set_current_state(TASK_INTERRUPTIBLE);
 	schedule();
-	rthal_apc_handler(0);
+	rthal_apc_handler(0, NULL);
     }
 
     __set_current_state(TASK_RUNNING);
@@ -486,8 +416,7 @@ void rthal_apc_kicker(unsigned virq, void *cookie)
  *
  * The HAL guarantees that any Linux kernel service which would be
  * callable from a regular Linux interrupt handler is also available
- * to APC handlers, including over PREEMPT_RT kernels exhibiting a
- * threaded IRQ model.
+ * to APC handlers.
  *
  * @param name is a symbolic name identifying the APC which will get
  * reported through the /proc/xenomai/apc interface. Passing NULL to
@@ -564,190 +493,6 @@ void rthal_apc_free(int apc)
 	clear_bit(apc, &rthal_apc_map);
 }
 
-#ifdef CONFIG_PROC_FS
-
-struct proc_dir_entry *rthal_proc_root;
-
-static int hal_read_proc(char *page,
-			 char **start,
-			 off_t off, int count, int *eof, void *data)
-{
-    int len, major, minor, patchlevel;
-
-    major = IPIPE_MAJOR_NUMBER;
-    minor = IPIPE_MINOR_NUMBER;
-    patchlevel = IPIPE_PATCH_NUMBER;
-
-    len = sprintf(page, "%d.%d-%.2d\n", major, minor, patchlevel);
-    len -= off;
-    if (len <= off + count)
-	*eof = 1;
-    *start = page + off;
-    if (len > count)
-	len = count;
-    if (len < 0)
-	len = 0;
-
-    return len;
-}
-
-static int faults_read_proc(char *page,
-			    char **start,
-			    off_t off, int count, int *eof, void *data)
-{
-    int len = 0, cpu, trap;
-    char *p = page;
-
-    p += sprintf(p, "TRAP ");
-
-    for_each_online_cpu(cpu) {
-	p += sprintf(p, "        CPU%d", cpu);
-    }
-
-    for (trap = 0; rthal_fault_labels[trap] != NULL; trap++) {
-
-	if (!*rthal_fault_labels[trap])
-	    continue;
-
-	p += sprintf(p, "\n%3d: ", trap);
-
-	for_each_online_cpu(cpu) {
-	    p += sprintf(p, "%12u", rthal_realtime_faults[cpu][trap]);
-	}
-
-	p += sprintf(p, "    (%s)", rthal_fault_labels[trap]);
-    }
-
-    p += sprintf(p, "\n");
-
-    len = p - page - off;
-    if (len <= off + count)
-	*eof = 1;
-    *start = page + off;
-    if (len > count)
-	len = count;
-    if (len < 0)
-	len = 0;
-
-    return len;
-}
-
-static int apc_read_proc(char *page,
-			 char **start,
-			 off_t off, int count, int *eof, void *data)
-{
-    int len = 0, cpu, apc;
-    char *p = page;
-
-    p += sprintf(p, "APC ");
-
-    for_each_online_cpu(cpu) {
-	p += sprintf(p, "         CPU%d", cpu);
-    }
-
-    for (apc = 0; apc < BITS_PER_LONG; apc++) {
-	if (!test_bit(apc, &rthal_apc_map))
-	    continue;           /* Not hooked. */
-
-	p += sprintf(p, "\n%3d: ", apc);
-
-	for_each_online_cpu(cpu) {
-	    p += sprintf(p, "%12lu", rthal_apc_table[apc].hits[cpu]);
-	}
-
-	if (rthal_apc_table[apc].name)
-	    p += sprintf(p, "    (%s)", rthal_apc_table[apc].name);
-    }
-
-    p += sprintf(p, "\n");
-
-    len = p - page - off;
-    if (len <= off + count)
-	*eof = 1;
-    *start = page + off;
-    if (len > count)
-	len = count;
-    if (len < 0)
-	len = 0;
-
-    return len;
-}
-
-struct proc_dir_entry *rthal_add_proc_leaf(const char *name,
-					   read_proc_t rdproc,
-					   write_proc_t wrproc,
-					   void *data,
-					   struct proc_dir_entry *parent)
-{
-	int mode = wrproc ? 0644 : 0444;
-	struct proc_dir_entry *entry;
-
-	entry = create_proc_entry(name, mode, parent);
-	if (entry == NULL)
-		return NULL;
-
-	entry->nlink = 1;
-	entry->data = data;
-	entry->read_proc = rdproc;
-	entry->write_proc = wrproc;
-	wrap_proc_dir_entry_owner(entry);
-
-	return entry;
-}
-EXPORT_SYMBOL_GPL(rthal_add_proc_leaf);
-
-struct proc_dir_entry *rthal_add_proc_seq(const char *name,
-					  struct file_operations *fops,
-					  size_t size,
-					  struct proc_dir_entry *parent)
-{
-	struct proc_dir_entry *entry;
-
-	entry = create_proc_entry(name, 0, parent);
-	if (entry == NULL)
-		return NULL;
-
-	entry->proc_fops = fops;
-	wrap_proc_dir_entry_owner(entry);
-
-	if (size)
-		entry->size = size;
-
-	return entry;
-}
-EXPORT_SYMBOL_GPL(rthal_add_proc_seq);
-
-static int rthal_proc_register(void)
-{
-	rthal_proc_root = create_proc_entry("xenomai", S_IFDIR, 0);
-	if (rthal_proc_root == NULL) {
-		printk(KERN_ERR "Xenomai: Unable to initialize /proc/xenomai.\n");
-		return -1;
-	}
-
-	wrap_proc_dir_entry_owner(rthal_proc_root);
-
-	rthal_add_proc_leaf("hal", &hal_read_proc, NULL, NULL, rthal_proc_root);
-	rthal_add_proc_leaf("faults",
-			    &faults_read_proc, NULL, NULL, rthal_proc_root);
-	rthal_add_proc_leaf("apc", &apc_read_proc, NULL, NULL, rthal_proc_root);
-
-	rthal_nmi_proc_register();
-
-	return 0;
-}
-
-static void rthal_proc_unregister(void)
-{
-    rthal_nmi_proc_unregister();
-    remove_proc_entry("hal", rthal_proc_root);
-    remove_proc_entry("faults", rthal_proc_root);
-    remove_proc_entry("apc", rthal_proc_root);
-    remove_proc_entry("xenomai", NULL);
-}
-
-#endif /* CONFIG_PROC_FS */
-
 int rthal_init(void)
 {
     int err;
@@ -796,29 +541,12 @@ int rthal_init(void)
 
     err = rthal_virtualize_irq(rthal_current_domain,
 			       rthal_apc_virq,
-			       &rthal_apc_trampoline,
+			       &rthal_apc_handler,
 			       NULL, NULL, IPIPE_HANDLE_MASK);
     if (err) {
 	printk(KERN_ERR "Xenomai: Failed to virtualize IRQ.\n");
 	goto out_free_irq;
     }
-#ifdef CONFIG_PREEMPT_RT
-    {
-	int cpu;
-	for_each_online_cpu(cpu) {
-	    rthal_apc_servers[cpu] =
-		kthread_create(&rthal_apc_thread, (void *)(unsigned long)cpu,
-			       "apc/%d", cpu);
-	    if (!rthal_apc_servers[cpu])
-		goto out_kthread_stop;
-	    wake_up_process(rthal_apc_servers[cpu]);
-	}
-    }
-#endif /* CONFIG_PREEMPT_RT */
-
-#ifdef CONFIG_PROC_FS
-    rthal_proc_register();
-#endif /* CONFIG_PROC_FS */
 
     err = rthal_register_domain(&rthal_domain,
 				"Xenomai",
@@ -838,25 +566,12 @@ int rthal_init(void)
 #endif
 	    printk(KERN_ERR "Xenomai: Domain registration failed (%d).\n", err);
 
-	goto out_proc_unregister;
+	goto fail;
     }
 
     return 0;
 
-  out_proc_unregister:
-#ifdef CONFIG_PROC_FS
-    rthal_proc_unregister();
-#endif
-#ifdef CONFIG_PREEMPT_RT
-  out_kthread_stop:
-    {
-	int cpu;
-	for_each_online_cpu(cpu) {
-	    if (rthal_apc_servers[cpu])
-		kthread_stop(rthal_apc_servers[cpu]);
-	}
-    }
-#endif /* CONFIG_PREEMPT_RT */
+  fail:
     rthal_virtualize_irq(rthal_current_domain, rthal_apc_virq, NULL, NULL, NULL,
 			 0);
 
@@ -872,22 +587,10 @@ int rthal_init(void)
 
 void rthal_exit(void)
 {
-#ifdef CONFIG_PROC_FS
-    rthal_proc_unregister();
-#endif /* CONFIG_PROC_FS */
-
     if (rthal_apc_virq) {
 	rthal_virtualize_irq(rthal_current_domain, rthal_apc_virq, NULL, NULL,
 			     NULL, 0);
 	rthal_free_virq(rthal_apc_virq);
-#ifdef CONFIG_PREEMPT_RT
-	{
-	    int cpu;
-	    for_each_online_cpu(cpu) {
-		kthread_stop(rthal_apc_servers[cpu]);
-	    }
-	}
-#endif /* CONFIG_PREEMPT_RT */
     }
 
     if (rthal_init_done)
@@ -1074,35 +777,31 @@ unsigned long long __rthal_generic_full_divmod64(unsigned long long a,
 
 /*@}*/
 
-EXPORT_SYMBOL(rthal_irq_request);
-EXPORT_SYMBOL(rthal_irq_release);
-EXPORT_SYMBOL(rthal_irq_enable);
-EXPORT_SYMBOL(rthal_irq_disable);
-EXPORT_SYMBOL(rthal_irq_end);
-EXPORT_SYMBOL(rthal_irq_host_request);
-EXPORT_SYMBOL(rthal_irq_host_release);
-EXPORT_SYMBOL(rthal_irq_affinity);
-EXPORT_SYMBOL(rthal_trap_catch);
-EXPORT_SYMBOL(rthal_timer_request);
-EXPORT_SYMBOL(rthal_timer_release);
-EXPORT_SYMBOL(rthal_timer_calibrate);
-EXPORT_SYMBOL(rthal_apc_alloc);
-EXPORT_SYMBOL(rthal_apc_free);
+EXPORT_SYMBOL_GPL(rthal_irq_request);
+EXPORT_SYMBOL_GPL(rthal_irq_release);
+EXPORT_SYMBOL_GPL(rthal_irq_enable);
+EXPORT_SYMBOL_GPL(rthal_irq_disable);
+EXPORT_SYMBOL_GPL(rthal_irq_end);
+EXPORT_SYMBOL_GPL(rthal_irq_host_request);
+EXPORT_SYMBOL_GPL(rthal_irq_host_release);
+EXPORT_SYMBOL_GPL(rthal_trap_catch);
+EXPORT_SYMBOL_GPL(rthal_timer_request);
+EXPORT_SYMBOL_GPL(rthal_timer_release);
+EXPORT_SYMBOL_GPL(rthal_timer_calibrate);
+EXPORT_SYMBOL_GPL(rthal_apc_alloc);
+EXPORT_SYMBOL_GPL(rthal_apc_free);
 
-EXPORT_SYMBOL(rthal_critical_enter);
-EXPORT_SYMBOL(rthal_critical_exit);
+EXPORT_SYMBOL_GPL(rthal_critical_enter);
+EXPORT_SYMBOL_GPL(rthal_critical_exit);
 
-EXPORT_SYMBOL(rthal_domain);
-EXPORT_SYMBOL(rthal_tunables);
-#ifdef CONFIG_PROC_FS
-EXPORT_SYMBOL(rthal_proc_root);
-#endif /* CONFIG_PROC_FS */
+EXPORT_SYMBOL_GPL(rthal_domain);
+EXPORT_SYMBOL_GPL(rthal_tunables);
 
-EXPORT_SYMBOL(rthal_init);
-EXPORT_SYMBOL(rthal_exit);
-EXPORT_SYMBOL(__rthal_generic_full_divmod64);
-EXPORT_SYMBOL(rthal_apc_virq);
-EXPORT_SYMBOL(rthal_apc_pending);
+EXPORT_SYMBOL_GPL(rthal_init);
+EXPORT_SYMBOL_GPL(rthal_exit);
+EXPORT_SYMBOL_GPL(__rthal_generic_full_divmod64);
+EXPORT_SYMBOL_GPL(rthal_apc_virq);
+EXPORT_SYMBOL_GPL(rthal_apc_pending);
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
 EXPORT_SYMBOL_GPL(kill_proc_info);
