@@ -137,10 +137,12 @@ void xnsched_init(struct xnsched *sched, int cpu)
 	strcpy(root_name, "ROOT");
 #endif
 	sched->status = 0;
+	sched->lflags = 0;
 	sched->inesting = 0;
 	sched->curr = &sched->rootcb;
 #ifdef CONFIG_XENO_OPT_PRIOCPL
 	xnlock_init(&sched->rpilock);
+	sched->rpistatus = 0;
 #endif
 	/*
 	 * No direct handler here since the host timer processing is
@@ -152,7 +154,9 @@ void xnsched_init(struct xnsched *sched, int cpu)
 	xntimer_set_name(&sched->htimer, htimer_name);
 	xntimer_set_sched(&sched->htimer, sched);
 	sched->zombie = NULL;
+#ifdef CONFIG_SMP
 	xnarch_cpus_clear(sched->resched);
+#endif
 
 	attr.flags = XNROOT | XNSTARTED | XNFPU;
 	attr.name = root_name;
@@ -685,68 +689,102 @@ struct xnpholder *nextmlq(struct xnsched_mlq *q, struct xnpholder *h)
 
 #endif /* CONFIG_XENO_OPT_SCALABLE_SCHED */
 
-#ifdef CONFIG_PROC_FS
+#ifdef CONFIG_XENO_OPT_VFILE
 
-#include <linux/seq_file.h>
+static struct xnvfile_directory schedclass_vfroot;
 
-static struct proc_dir_entry *schedclass_proc_root;
-
-struct sched_seq_iterator {
+struct vfile_sched_priv {
+	struct xnholder *curr;
 	xnticks_t start_time;
-	int nentries;
-	struct sched_seq_info {
-		int cpu;
-		pid_t pid;
-		char name[XNOBJECT_NAME_LEN];
-		char timebase[XNOBJECT_NAME_LEN];
-		char sched_class[XNOBJECT_NAME_LEN];
-		int cprio;
-		int dnprio;
-		int periodic;
-		xnticks_t timeout;
-		xnflags_t state;
-	} sched_info[1];
 };
 
-static void *sched_seq_start(struct seq_file *seq, loff_t *pos)
+struct vfile_sched_data {
+	int cpu;
+	pid_t pid;
+	char name[XNOBJECT_NAME_LEN];
+	char timebase[XNOBJECT_NAME_LEN];
+	char sched_class[XNOBJECT_NAME_LEN];
+	int cprio;
+	int dnprio;
+	int periodic;
+	xnticks_t timeout;
+	xnflags_t state;
+};
+
+static struct xnvfile_snapshot_ops vfile_sched_ops;
+
+static struct xnvfile_snapshot sched_vfile = {
+	.privsz = sizeof(struct vfile_sched_priv),
+	.datasz = sizeof(struct vfile_sched_data),
+	.tag = &nkpod_struct.threadlist_tag,
+	.ops = &vfile_sched_ops,
+};
+
+static int vfile_sched_rewind(struct xnvfile_snapshot_iterator *it)
 {
-	struct sched_seq_iterator *iter = seq->private;
+	struct vfile_sched_priv *priv = xnvfile_iterator_priv(it);
 
-	if (*pos > iter->nentries)
-		return NULL;
+	priv->curr = getheadq(&nkpod->threadq);
+	priv->start_time = xntbase_get_jiffies(&nktbase);
 
-	if (*pos == 0)
-		return SEQ_START_TOKEN;
-
-	return iter->sched_info + *pos - 1;
+	return countq(&nkpod->threadq);
 }
 
-static void *sched_seq_next(struct seq_file *seq, void *v, loff_t *pos)
+static int vfile_sched_next(struct xnvfile_snapshot_iterator *it, void *data)
 {
-	struct sched_seq_iterator *iter = seq->private;
+	struct vfile_sched_priv *priv = xnvfile_iterator_priv(it);
+	struct vfile_sched_data *p = data;
+	xnticks_t timeout, period;
+	struct xnthread *thread;
 
-	++*pos;
+	if (priv->curr == NULL)
+		return 0;	/* All done. */
 
-	if (*pos > iter->nentries)
-		return NULL;
+	thread = link2thread(priv->curr, glink);
+	priv->curr = nextq(&nkpod->threadq, priv->curr);
 
-	return iter->sched_info + *pos - 1;
+	p->cpu = xnsched_cpu(thread->sched);
+	p->pid = xnthread_user_pid(thread);
+	memcpy(p->name, thread->name, sizeof(p->name));
+	p->cprio = thread->cprio;
+	p->dnprio = xnthread_get_denormalized_prio(thread, thread->cprio);
+	p->state = xnthread_state_flags(thread);
+	memcpy(p->timebase, xntbase_name(xnthread_time_base(thread)),
+	       sizeof(p->timebase));
+	xnobject_copy_name(p->sched_class, thread->sched_class->name);
+	period = xnthread_get_period(thread);
+	timeout = xnthread_get_timeout(thread, priv->start_time);
+	/*
+	 * Here we cheat: thread is periodic and the sampling rate may
+	 * be high, so it is indeed possible that the next tick date
+	 * from the ptimer progresses fast enough while we are busy
+	 * collecting output data in this loop, so that next_date -
+	 * start_time > period. In such a case, we simply ceil the
+	 * value to period to keep the result meaningful, even if not
+	 * necessarily accurate. But what does accuracy mean when the
+	 * sampling frequency is high, and the way to read it has to
+	 * go through the vfile interface anyway?
+	 */
+	if (period > 0 && period < timeout &&
+	    !xntimer_running_p(&thread->rtimer))
+		timeout = period;
+	p->timeout = timeout;
+	p->periodic = xntbase_periodic_p(xnthread_time_base(thread));
+
+	return 1;
 }
 
-static void sched_seq_stop(struct seq_file *seq, void *v)
-{
-}
-
-static int sched_seq_show(struct seq_file *seq, void *v)
+static int vfile_sched_show(struct xnvfile_snapshot_iterator *it, void *data)
 {
+	struct vfile_sched_data *p = data;
 	char sbuf[64], pbuf[16], tbuf[16];
 
-	if (v == SEQ_START_TOKEN)
-		seq_printf(seq, "%-3s  %-6s %-5s  %-8s %-8s  %-10s %-10s %s\n",
-			   "CPU", "PID", "CLASS", "PRI", "TIMEOUT", "TIMEBASE", "STAT", "NAME");
+	if (p == NULL)
+		xnvfile_printf(it,
+			       "%-3s  %-6s %-5s  %-8s %-8s  %-10s %-10s %s\n",
+			       "CPU", "PID", "CLASS", "PRI", "TIMEOUT",
+			       "TIMEBASE", "STAT", "NAME");
 	else {
-		struct sched_seq_info *p = v;
-
 		if (p->cprio != p->dnprio)
 			snprintf(pbuf, sizeof(pbuf), "%3d(%d)",
 				 p->cprio, p->dnprio);
@@ -756,419 +794,249 @@ static int sched_seq_show(struct seq_file *seq, void *v)
 		xntimer_format_time(p->timeout, p->periodic, tbuf, sizeof(tbuf));
 		xnthread_format_status(p->state, sbuf, sizeof(sbuf));
 
-		seq_printf(seq, "%3u  %-6d %-5s  %-8s %-8s  %-10s %-10s %s\n",
-			   p->cpu,
-			   p->pid,
-			   p->sched_class,
-			   pbuf,
-			   tbuf,
-			   p->timebase,
-			   sbuf,
-			   p->name);
+		xnvfile_printf(it,
+			       "%3u  %-6d %-5s  %-8s %-8s  %-10s %-10s %s\n",
+			       p->cpu,
+			       p->pid,
+			       p->sched_class,
+			       pbuf,
+			       tbuf,
+			       p->timebase,
+			       sbuf,
+			       p->name);
 	}
 
 	return 0;
 }
 
-static struct seq_operations sched_op = {
-	.start = &sched_seq_start,
-	.next = &sched_seq_next,
-	.stop = &sched_seq_stop,
-	.show = &sched_seq_show
-};
-
-static int sched_seq_open(struct inode *inode, struct file *file)
-{
-	struct sched_seq_iterator *iter = NULL;
-	xnticks_t period, timeout;
-	struct xnholder *holder;
-	struct sched_seq_info *p;
-	struct seq_file *seq;
-	int err, count, rev;
-	spl_t s;
-
-	if (!xnpod_active_p())
-		return -ESRCH;
-
-	xnlock_get_irqsave(&nklock, s);
-
-      restart:
-	rev = nkpod->threadq_rev;
-	count = countq(&nkpod->threadq);	/* Cannot be empty (ROOT) */
-	holder = getheadq(&nkpod->threadq);
-
-	xnlock_put_irqrestore(&nklock, s);
-
-	if (iter)
-		kfree(iter);
-	iter = kmalloc(sizeof(*iter)
-		       + (count - 1) * sizeof(struct sched_seq_info),
-		       GFP_KERNEL);
-	if (!iter)
-		return -ENOMEM;
-
-	err = seq_open(file, &sched_op);
-
-	if (err) {
-		kfree(iter);
-		return err;
-	}
-
-	iter->nentries = 0;
-	iter->start_time = xntbase_get_jiffies(&nktbase);
-
-	/*
-	 * Take a snapshot element-wise, restart if something changes
-	 * underneath us.
-	 */
-	while (holder) {
-		xnthread_t *thread;
-		int n;
-
-		xnlock_get_irqsave(&nklock, s);
-
-		if (nkpod->threadq_rev != rev)
-			goto restart;
-
-		rev = nkpod->threadq_rev;
-		thread = link2thread(holder, glink);
-		n = iter->nentries++;
-		p = iter->sched_info + n;
-
-		p->cpu = xnsched_cpu(thread->sched);
-		p->pid = xnthread_user_pid(thread);
-		memcpy(p->name, thread->name, sizeof(p->name));
-		p->cprio = thread->cprio;
-		p->dnprio = xnthread_get_denormalized_prio(thread, thread->cprio);
-		p->state = xnthread_state_flags(thread);
-		memcpy(p->timebase, xntbase_name(xnthread_time_base(thread)),
-		       sizeof(p->timebase));
-		xnobject_copy_name(p->sched_class, thread->sched_class->name);
-		period = xnthread_get_period(thread);
-		timeout = xnthread_get_timeout(thread, iter->start_time);
-		/*
-		 * Here we cheat: thread is periodic and the sampling
-		 * rate may be high, so it is indeed possible that the
-		 * next tick date from the ptimer progresses fast
-		 * enough while we are busy collecting output data in
-		 * this loop, so that next_date - start_time >
-		 * period. In such a case, we simply ceil the value to
-		 * period to keep the result meaningful, even if not
-		 * necessarily accurate. But what does accuracy mean
-		 * when the sampling frequency is high, and the way to
-		 * read it has to go through the /proc interface
-		 * anyway?
-		 */
-		if (period > 0 && period < timeout &&
-		    !xntimer_running_p(&thread->rtimer))
-			timeout = period;
-		p->timeout = timeout;
-		p->periodic = xntbase_periodic_p(xnthread_time_base(thread));
-
-		holder = nextq(&nkpod->threadq, holder);
-		xnlock_put_irqrestore(&nklock, s);
-	}
-
-	seq = file->private_data;
-	seq->private = iter;
-
-	return 0;
-}
-
-static struct file_operations sched_seq_operations = {
-	.owner = THIS_MODULE,
-	.open = sched_seq_open,
-	.read = seq_read,
-	.llseek = seq_lseek,
-	.release = seq_release_private,
+static struct xnvfile_snapshot_ops vfile_sched_ops = {
+	.rewind = vfile_sched_rewind,
+	.next = vfile_sched_next,
+	.show = vfile_sched_show,
 };
 
 #ifdef CONFIG_XENO_OPT_STATS
 
-struct stat_seq_iterator {
-	int nentries;
-	struct stat_seq_info {
-		int cpu;
-		pid_t pid;
-		xnflags_t state;
-		char name[XNOBJECT_NAME_LEN];
-		unsigned long ssw;
-		unsigned long csw;
-		unsigned long pf;
-		xnticks_t exectime_period;
-		xnticks_t account_period;
-		xnticks_t exectime_total;
-	} stat_info[1];
+struct vfile_stat_priv {
+	int irq;
+	struct xnholder *curr;
+	struct xnintr_iterator intr_it;
 };
 
-static void *stat_seq_start(struct seq_file *seq, loff_t *pos)
+struct vfile_stat_data {
+	int cpu;
+	pid_t pid;
+	xnflags_t state;
+	char name[XNOBJECT_NAME_LEN];
+	unsigned long ssw;
+	unsigned long csw;
+	unsigned long pf;
+	xnticks_t exectime_period;
+	xnticks_t account_period;
+	xnticks_t exectime_total;
+};
+
+static struct xnvfile_snapshot_ops vfile_stat_ops;
+
+static struct xnvfile_snapshot stat_vfile = {
+	.privsz = sizeof(struct vfile_stat_priv),
+	.datasz = sizeof(struct vfile_stat_data),
+	.tag = &nkpod_struct.threadlist_tag,
+	.ops = &vfile_stat_ops,
+};
+
+static int vfile_stat_rewind(struct xnvfile_snapshot_iterator *it)
 {
-	struct stat_seq_iterator *iter = seq->private;
+	struct vfile_stat_priv *priv = xnvfile_iterator_priv(it);
+	int irqnr;
 
-	if (*pos > iter->nentries)
-		return NULL;
+	/*
+	 * The activity numbers on each valid interrupt descriptor are
+	 * grouped under a pseudo-thread.
+	 */
+	priv->curr = getheadq(&nkpod->threadq);
+	priv->irq = 0;
+	irqnr = xnintr_query_init(&priv->intr_it) * XNARCH_NR_CPUS;
 
-	if (*pos == 0)
-		return SEQ_START_TOKEN;
-
-	return iter->stat_info + *pos - 1;
+	return irqnr + countq(&nkpod->threadq);
 }
 
-static void *stat_seq_next(struct seq_file *seq, void *v, loff_t *pos)
+static int vfile_stat_next(struct xnvfile_snapshot_iterator *it, void *data)
 {
-	struct stat_seq_iterator *iter = seq->private;
+	struct vfile_stat_priv *priv = xnvfile_iterator_priv(it);
+	struct vfile_stat_data *p = data;
+	struct xnthread *thread;
+	struct xnsched *sched;
+	xnticks_t period;
+	int ret;
 
-	++*pos;
+	if (priv->curr == NULL)
+		/*
+		 * We are done with actual threads, scan interrupt
+		 * descriptors.
+		 */
+		goto scan_irqs;
 
-	if (*pos > iter->nentries)
-		return NULL;
+	thread = link2thread(priv->curr, glink);
+	priv->curr = nextq(&nkpod->threadq, priv->curr);
 
-	return iter->stat_info + *pos - 1;
-}
+	sched = thread->sched;
+	p->cpu = xnsched_cpu(sched);
+	p->pid = xnthread_user_pid(thread);
+	memcpy(p->name, thread->name, sizeof(p->name));
+	p->state = xnthread_state_flags(thread);
+	p->ssw = xnstat_counter_get(&thread->stat.ssw);
+	p->csw = xnstat_counter_get(&thread->stat.csw);
+	p->pf = xnstat_counter_get(&thread->stat.pf);
 
-static void stat_seq_stop(struct seq_file *seq, void *v)
-{
-}
+	period = sched->last_account_switch - thread->stat.lastperiod.start;
+	if (period == 0 && thread == sched->curr) {
+		p->exectime_period = 1;
+		p->account_period = 1;
+	} else {
+		p->exectime_period = thread->stat.account.total -
+			thread->stat.lastperiod.total;
+		p->account_period = period;
+	}
+	p->exectime_total = thread->stat.account.total;
+	thread->stat.lastperiod.total = thread->stat.account.total;
+	thread->stat.lastperiod.start = sched->last_account_switch;
 
-static int __stat_seq_open(struct inode *inode,
-			   struct file *file, struct seq_operations *ops)
-{
-	struct stat_seq_iterator *iter = NULL;
-	struct stat_seq_info *stat_info;
-	int err, count, thrq_rev, irq;
-	xnintr_iterator_t intr_iter;
-	struct seq_file *seq;
-	xnholder_t *holder;
-	int intr_count;
-	spl_t s;
+	return 1;
 
-	if (!xnpod_active_p())
-		return -ESRCH;
+scan_irqs:
+	if (priv->irq >= XNARCH_NR_IRQS)
+		return 0;	/* All done. */
 
-      restart_unlocked:
-	xnlock_get_irqsave(&nklock, s);
-
-      restart:
-	count = countq(&nkpod->threadq);	/* Cannot be empty (ROOT) */
-	holder = getheadq(&nkpod->threadq);
-	thrq_rev = nkpod->threadq_rev;
-
-	xnlock_put_irqrestore(&nklock, s);
-
-	intr_count = xnintr_query_init(&intr_iter);
-	count += intr_count * RTHAL_NR_CPUS;
-
-	if (iter)
-		kfree(iter);
-	iter = kmalloc(sizeof(*iter)
-		       + (count - 1) * sizeof(struct stat_seq_info),
-		       GFP_KERNEL);
-	if (!iter)
-		return -ENOMEM;
-
-	err = seq_open(file, ops);
-
-	if (err) {
-		kfree(iter);
-		return err;
+	ret = xnintr_query_next(priv->irq++, &priv->intr_it, p->name);
+	if (ret) {
+		if (ret == -EAGAIN)
+			xnvfile_touch(it->vfile); /* force rewind. */
+		return VFILE_SEQ_SKIP;
 	}
 
-	iter->nentries = 0;
+	if (!xnarch_cpu_supported(priv->intr_it.cpu))
+		return VFILE_SEQ_SKIP;
 
-	/* Take a snapshot element-wise, restart if something changes
-	   underneath us. */
+	p->cpu = priv->intr_it.cpu;
+	p->csw = priv->intr_it.hits;
+	p->exectime_period = priv->intr_it.exectime_period;
+	p->account_period = priv->intr_it.account_period;
+	p->exectime_total = priv->intr_it.exectime_total;
+	p->pid = 0;
+	p->state =  0;
+	p->ssw = 0;
+	p->pf = 0;
 
-	while (holder) {
-		xnthread_t *thread;
-		xnsched_t *sched;
-		xnticks_t period;
-
-		xnlock_get_irqsave(&nklock, s);
-
-		if (nkpod->threadq_rev != thrq_rev)
-			goto restart;
-
-		thread = link2thread(holder, glink);
-		stat_info = &iter->stat_info[iter->nentries++];
-
-		sched = thread->sched;
-		stat_info->cpu = xnsched_cpu(sched);
-		stat_info->pid = xnthread_user_pid(thread);
-		memcpy(stat_info->name, thread->name,
-		       sizeof(stat_info->name));
-		stat_info->state = xnthread_state_flags(thread);
-		stat_info->ssw = xnstat_counter_get(&thread->stat.ssw);
-		stat_info->csw = xnstat_counter_get(&thread->stat.csw);
-		stat_info->pf = xnstat_counter_get(&thread->stat.pf);
-
-		period = sched->last_account_switch - thread->stat.lastperiod.start;
-		if (!period && thread == sched->curr) {
-			stat_info->exectime_period = 1;
-			stat_info->account_period = 1;
-		} else {
-			stat_info->exectime_period = thread->stat.account.total -
-				thread->stat.lastperiod.total;
-			stat_info->account_period = period;
-		}
-		stat_info->exectime_total = thread->stat.account.total;
-		thread->stat.lastperiod.total = thread->stat.account.total;
-		thread->stat.lastperiod.start = sched->last_account_switch;
-
-		holder = nextq(&nkpod->threadq, holder);
-
-		xnlock_put_irqrestore(&nklock, s);
-	}
-
-	/* Iterate over all IRQ numbers, ... */
-	for (irq = 0; irq < XNARCH_NR_IRQS; irq++)
-		/* ...over all shared IRQs on all CPUs */
-		while (1) {
-			stat_info = &iter->stat_info[iter->nentries];
-
-			err = xnintr_query_next(irq, &intr_iter,
-						stat_info->name);
-			if (err) {
-				if (err == -EAGAIN)
-					goto restart_unlocked;
-				break; /* line unused or end of chain */
-			}
-
-			if (xnarch_cpu_supported(intr_iter.cpu)) {
-				stat_info->cpu = intr_iter.cpu;
-				stat_info->csw = intr_iter.hits;
-				stat_info->exectime_period =
-					intr_iter.exectime_period;
-				stat_info->account_period =
-					intr_iter.account_period;
-				stat_info->exectime_total =
-					intr_iter.exectime_total;
-				stat_info->pid = 0;
-				stat_info->state =  0;
-				stat_info->ssw = 0;
-				stat_info->pf = 0;
-
-				iter->nentries++;
-			}
-		}
-
-	seq = file->private_data;
-	seq->private = iter;
-
-	return 0;
+	return 1;
 }
 
-static int stat_seq_show(struct seq_file *seq, void *v)
+static int vfile_stat_show(struct xnvfile_snapshot_iterator *it, void *data)
 {
-	if (v == SEQ_START_TOKEN)
-		seq_printf(seq, "%-3s  %-6s %-10s %-10s %-4s  %-8s  %5s"
-			   "  %s\n",
-			   "CPU", "PID", "MSW", "CSW", "PF", "STAT", "%CPU",
-			   "NAME");
+	struct vfile_stat_data *p = data;
+	int usage = 0;
+
+	if (p == NULL)
+		xnvfile_printf(it,
+			       "%-3s  %-6s %-10s %-10s %-4s  %-8s  %5s"
+			       "  %s\n",
+			       "CPU", "PID", "MSW", "CSW", "PF", "STAT", "%CPU",
+			       "NAME");
 	else {
-		struct stat_seq_info *p = v;
-		int usage = 0;
-
 		if (p->account_period) {
-			while (p->account_period > 0xFFFFFFFF) {
+			while (p->account_period > 0xffffffffUL) {
 				p->exectime_period >>= 16;
 				p->account_period >>= 16;
 			}
-			usage =
-			    xnarch_ulldiv(p->exectime_period * 1000LL +
-					  (p->account_period >> 1),
-					  p->account_period, NULL);
+			usage = xnarch_ulldiv(p->exectime_period * 1000LL +
+					      (p->account_period >> 1),
+					      p->account_period, NULL);
 		}
-		seq_printf(seq, "%3u  %-6d %-10lu %-10lu %-4lu  %.8lx  %3u.%u"
-			   "  %s\n",
-			   p->cpu, p->pid, p->ssw, p->csw, p->pf, p->state,
-			   usage / 10, usage % 10, p->name);
+		xnvfile_printf(it,
+			       "%3u  %-6d %-10lu %-10lu %-4lu  %.8lx  %3u.%u"
+			       "  %s\n",
+			       p->cpu, p->pid, p->ssw, p->csw, p->pf, p->state,
+			       usage / 10, usage % 10, p->name);
 	}
 
 	return 0;
 }
 
-static struct seq_operations stat_op = {
-	.start = &stat_seq_start,
-	.next = &stat_seq_next,
-	.stop = &stat_seq_stop,
-	.show = &stat_seq_show
-};
-
-static int stat_seq_open(struct inode *inode, struct file *file)
+static int vfile_acct_show(struct xnvfile_snapshot_iterator *it, void *data)
 {
-	return __stat_seq_open(inode, file, &stat_op);
-}
+	struct vfile_stat_data *p = data;
 
-static struct file_operations stat_seq_operations = {
-	.owner = THIS_MODULE,
-	.open = stat_seq_open,
-	.read = seq_read,
-	.llseek = seq_lseek,
-	.release = seq_release_private,
-};
-
-static int acct_seq_show(struct seq_file *seq, void *v)
-{
-	struct stat_seq_info *p;
-
-	if (v == SEQ_START_TOKEN)
+	if (p == NULL)
 		return 0;
-	/*
-	 * Dump per-thread data.
-	 */
-	p = v;
 
-	seq_printf(seq, "%u %d %lu %lu %lu %.8lx %Lu %Lu %Lu %s\n",
-		   p->cpu, p->pid, p->ssw, p->csw, p->pf, p->state,
-		   xnarch_tsc_to_ns(p->account_period), xnarch_tsc_to_ns(p->exectime_period),
-		   xnarch_tsc_to_ns(p->exectime_total), p->name);
+	xnvfile_printf(it, "%u %d %lu %lu %lu %.8lx %Lu %Lu %Lu %s\n",
+		       p->cpu, p->pid, p->ssw, p->csw, p->pf, p->state,
+		       xnarch_tsc_to_ns(p->account_period),
+		       xnarch_tsc_to_ns(p->exectime_period),
+		       xnarch_tsc_to_ns(p->exectime_total),
+		       p->name);
 
 	return 0;
 }
 
-static struct seq_operations acct_op = {
-	.start = &stat_seq_start,
-	.next = &stat_seq_next,
-	.stop = &stat_seq_stop,
-	.show = &acct_seq_show
+static struct xnvfile_snapshot_ops vfile_stat_ops = {
+	.rewind = vfile_stat_rewind,
+	.next = vfile_stat_next,
+	.show = vfile_stat_show,
 };
 
-static int acct_seq_open(struct inode *inode, struct file *file)
-{
-	return __stat_seq_open(inode, file, &acct_op);
-}
+/*
+ * An accounting vfile is a thread statistics vfile in disguise with a
+ * different output format, which is parser-friendly.
+ */
+static struct xnvfile_snapshot_ops vfile_acct_ops;
 
-static struct file_operations acct_seq_operations = {
-	.owner = THIS_MODULE,
-	.open = acct_seq_open,
-	.read = seq_read,
-	.llseek = seq_lseek,
-	.release = seq_release_private,
+static struct xnvfile_snapshot acct_vfile = {
+	.privsz = sizeof(struct vfile_stat_priv),
+	.datasz = sizeof(struct vfile_stat_data),
+	.tag = &nkpod_struct.threadlist_tag,
+	.ops = &vfile_acct_ops,
+};
+
+static struct xnvfile_snapshot_ops vfile_acct_ops = {
+	.rewind = vfile_stat_rewind,
+	.next = vfile_stat_next,
+	.show = vfile_acct_show,
 };
 
 #endif /* CONFIG_XENO_OPT_STATS */
 
-void xnsched_init_proc(void)
+int xnsched_init_proc(void)
 {
 	struct xnsched_class *p;
+	int ret;
 
-	rthal_add_proc_seq("sched", &sched_seq_operations, 0, rthal_proc_root);
-	schedclass_proc_root =
-		create_proc_entry("schedclasses", S_IFDIR, rthal_proc_root);
+	ret = xnvfile_init_snapshot("sched", &sched_vfile, &nkvfroot);
+	if (ret)
+		return ret;
+
+	ret = xnvfile_init_dir("schedclasses", &schedclass_vfroot, &nkvfroot);
+	if (ret)
+		return ret;
 
 	for_each_xnsched_class(p) {
-		if (p->sched_init_proc == NULL)
-			continue;
-		p->proc = create_proc_entry(p->name, S_IFDIR,
-					    schedclass_proc_root);
-		if (p->proc)
-			p->sched_init_proc(p->proc);
+		if (p->sched_init_vfile) {
+			ret = p->sched_init_vfile(p, &schedclass_vfroot);
+			if (ret)
+				return ret;
+		}
 	}
 
 #ifdef CONFIG_XENO_OPT_STATS
-	rthal_add_proc_seq("stat", &stat_seq_operations, 0, rthal_proc_root);
-	rthal_add_proc_seq("acct", &acct_seq_operations, 0, rthal_proc_root);
+	ret = xnvfile_init_snapshot("stat", &stat_vfile, &nkvfroot);
+	if (ret)
+		return ret;
+	ret = xnvfile_init_snapshot("acct", &acct_vfile, &nkvfroot);
+	if (ret)
+		return ret;
 #endif /* CONFIG_XENO_OPT_STATS */
+
+	return 0;
 }
 
 void xnsched_cleanup_proc(void)
@@ -1176,19 +1044,16 @@ void xnsched_cleanup_proc(void)
 	struct xnsched_class *p;
 
 	for_each_xnsched_class(p) {
-		if (p->proc == NULL)
-			continue;
-		if (p->sched_cleanup_proc)
-			p->sched_cleanup_proc(p->proc);
-		remove_proc_entry(p->name, schedclass_proc_root);
+		if (p->sched_cleanup_vfile)
+			p->sched_cleanup_vfile(p);
 	}
 
 #ifdef CONFIG_XENO_OPT_STATS
-	remove_proc_entry("acct", rthal_proc_root);
-	remove_proc_entry("stat", rthal_proc_root);
+	xnvfile_destroy_snapshot(&acct_vfile);
+	xnvfile_destroy_snapshot(&stat_vfile);
 #endif /* CONFIG_XENO_OPT_STATS */
-	remove_proc_entry("schedclasses", rthal_proc_root);
-	remove_proc_entry("sched", rthal_proc_root);
+	xnvfile_destroy_dir(&schedclass_vfroot);
+	xnvfile_destroy_snapshot(&sched_vfile);
 }
 
-#endif /* CONFIG_PROC_FS */
+#endif /* CONFIG_XENO_OPT_VFILE */
