@@ -2,7 +2,7 @@
  * @file
  * This file is part of the Xenomai project.
  *
- * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org> 
+ * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -51,100 +51,149 @@
 #include <native/task.h>
 #include <native/mutex.h>
 
-#ifdef CONFIG_PROC_FS
+#ifdef CONFIG_XENO_OPT_VFILE
 
-static int __mutex_read_proc(char *page,
-			     char **start,
-			     off_t off, int count, int *eof, void *data)
+struct vfile_priv {
+	struct xnpholder *curr;
+	char owner[XNOBJECT_NAME_LEN];
+};
+
+struct vfile_data {
+	char name[XNOBJECT_NAME_LEN];
+};
+
+static int vfile_rewind(struct xnvfile_snapshot_iterator *it)
 {
-	RT_MUTEX *mutex = (RT_MUTEX *)data;
+	struct vfile_priv *priv = xnvfile_iterator_priv(it);
+	RT_MUTEX *mutex = xnvfile_priv(it->vfile);
+	struct xnthread *owner;
 #ifdef CONFIG_XENO_FASTSYNCH
 	xnhandle_t lock_state;
-#endif /* CONFIG_XENO_FASTSYNCH */
-	xnthread_t *owner;
-	char *p = page;
-	int len;
-	spl_t s;
+#endif
+	mutex = xeno_h2obj_validate(mutex, XENO_MUTEX_MAGIC, RT_MUTEX);
+	if (mutex == NULL)
+		return -EIDRM;
 
-	xnlock_get_irqsave(&nklock, s);
-
-#ifndef CONFIG_XENO_FASTSYNCH
-	owner = xnsynch_owner(&mutex->synch_base);
-#else /* CONFIG_XENO_FASTSYNCH */
+#ifdef CONFIG_XENO_FASTSYNCH
 	lock_state = xnarch_atomic_get(mutex->synch_base.fastlock);
-
 	owner = (lock_state == XN_NO_HANDLE) ? NULL :
 		xnthread_lookup(xnsynch_fast_mask_claimed(lock_state));
 
-	if (!owner && lock_state != XN_NO_HANDLE)
-		p += sprintf(p, "=<DAMAGED HANDLE!>");
+	if (owner == NULL && lock_state != XN_NO_HANDLE)
+		strncpy(priv->owner, "<DAMAGED HANDLE>",
+			sizeof(priv->owner));
 	else
-#endif /* CONFIG_XENO_FASTSYNCH */
-	if (owner) {
-		/* Locked mutex -- dump owner and waiters, if any. */
-		xnpholder_t *holder;
+#else /* !CONFIG_XENO_FASTSYNCH */
+	owner = xnsynch_owner(&mutex->synch_base);
+#endif /* !CONFIG_XENO_FASTSYNCH */
+	if (owner)
+		strncpy(priv->owner, xnthread_name(owner),
+			sizeof(priv->owner));
+	else
+		*priv->owner = 0;
 
-		p += sprintf(p, "=locked by %s\n", xnthread_name(owner));
+	priv->curr = getheadpq(xnsynch_wait_queue(&mutex->synch_base));
 
-		holder = getheadpq(xnsynch_wait_queue(&mutex->synch_base));
-
-		while (holder) {
-			xnthread_t *sleeper = link2thread(holder, plink);
-
-			p += sprintf(p, "+%s\n", xnthread_name(sleeper));
-			holder = nextpq(xnsynch_wait_queue(&mutex->synch_base),
-					holder);
-		}
-	} else
-		/* Mutex unlocked. */
-		p += sprintf(p, "=unlocked\n");
-
-	xnlock_put_irqrestore(&nklock, s);
-
-	len = (p - page) - off;
-	if (len <= off + count)
-		*eof = 1;
-	*start = page + off;
-	if (len > count)
-		len = count;
-	if (len < 0)
-		len = 0;
-
-	return len;
+	return xnsynch_nsleepers(&mutex->synch_base);
 }
 
-extern xnptree_t __native_ptree;
-
-static xnpnode_t __mutex_pnode = {
-
-	.dir = NULL,
-	.type = "mutexes",
-	.entries = 0,
-	.read_proc = &__mutex_read_proc,
-	.write_proc = NULL,
-	.root = &__native_ptree,
-};
-
-#else /* !CONFIG_PROC_FS */
-
-static xnpnode_t __mutex_pnode = {
-
-	.type = "mutexes"
-};
-
-#endif /* !CONFIG_PROC_FS */
-
-int rt_mutex_create_inner(RT_MUTEX *mutex, const char *name,
-			  xnarch_atomic_t *fastlock)
+static int vfile_next(struct xnvfile_snapshot_iterator *it, void *data)
 {
+	struct vfile_priv *priv = xnvfile_iterator_priv(it);
+	RT_MUTEX *mutex = xnvfile_priv(it->vfile);
+	struct vfile_data *p = data;
+	struct xnthread *thread;
+
+	if (priv->curr == NULL)
+		return 0;	/* We are done. */
+
+	/* Fetch current waiter, advance list cursor. */
+	thread = link2thread(priv->curr, plink);
+	priv->curr = nextpq(xnsynch_wait_queue(&mutex->synch_base),
+			    priv->curr);
+	/* Collect thread name to be output in ->show(). */
+	strncpy(p->name, xnthread_name(thread), sizeof(p->name));
+
+	return 1;
+}
+
+static int vfile_show(struct xnvfile_snapshot_iterator *it, void *data)
+{
+	struct vfile_priv *priv = xnvfile_iterator_priv(it);
+	struct vfile_data *p = data;
+
+	if (p == NULL) {	/* Dump header. */
+		if (*priv->owner == 0)
+			/* Unlocked mutex. */
+			xnvfile_printf(it, "=unlocked\n");
+		else {
+			xnvfile_printf(it, "=locked by %.*s\n",
+				       (int)sizeof(priv->owner), priv->owner);
+			if (it->nrdata > 0)
+				/* Mutex is contended -- dump waiters */
+				xnvfile_printf(it, "--------------------\n");
+		}
+	} else
+		xnvfile_printf(it, "%.*s\n",
+			       (int)sizeof(p->name), p->name);
+
+	return 0;
+}
+
+static struct xnvfile_snapshot_ops vfile_ops = {
+	.rewind = vfile_rewind,
+	.next = vfile_next,
+	.show = vfile_show,
+};
+
+extern struct xnptree __native_ptree;
+
+static struct xnpnode_snapshot __mutex_pnode = {
+	.node = {
+		.dirname = "mutexes",
+		.root = &__native_ptree,
+		.ops = &xnregistry_vfsnap_ops,
+	},
+	.vfile = {
+		.privsz = sizeof(struct vfile_priv),
+		.datasz = sizeof(struct vfile_data),
+		.ops = &vfile_ops,
+	},
+};
+
+#else /* !CONFIG_XENO_OPT_VFILE */
+
+static struct xnpnode_snapshot __mutex_pnode = {
+	.node = {
+		.dirname = "mutexes",
+	},
+};
+
+#endif /* !CONFIG_XENO_OPT_VFILE */
+
+int rt_mutex_create_inner(RT_MUTEX *mutex, const char *name, int global)
+{
+	xnflags_t flags = XNSYNCH_PRIO | XNSYNCH_PIP | XNSYNCH_OWNER;
+	xnarch_atomic_t *fastlock = NULL;
 	int err = 0;
 	spl_t s;
 
 	if (xnpod_asynch_p())
 		return -EPERM;
 
-	xnsynch_init(&mutex->synch_base,
-		     XNSYNCH_PRIO | XNSYNCH_PIP | XNSYNCH_OWNER, fastlock);
+#ifdef CONFIG_XENO_FASTSYNCH
+	/* Allocate lock memory for in-kernel use */
+	fastlock = xnheap_alloc(&xnsys_ppd_get(global)->sem_heap,
+				sizeof(*fastlock));
+
+	if (!fastlock)
+		return -ENOMEM;
+
+	if (global)
+		flags |= RT_MUTEX_EXPORTED;
+#endif /* CONFIG_XENO_FASTSYNCH */
+
+	xnsynch_init(&mutex->synch_base, flags, fastlock);
 	mutex->handle = 0;	/* i.e. (still) unregistered mutex. */
 	mutex->magic = XENO_MUTEX_MAGIC;
 	mutex->lockcnt = 0;
@@ -166,10 +215,10 @@ int rt_mutex_create_inner(RT_MUTEX *mutex, const char *name,
 	 */
 	if (name) {
 		err = xnregistry_enter(mutex->name, mutex, &mutex->handle,
-				       &__mutex_pnode);
+				       &__mutex_pnode.node);
 
 		if (err)
-			rt_mutex_delete_inner(mutex);
+			rt_mutex_delete(mutex);
 	}
 
 	return err;
@@ -219,63 +268,7 @@ int rt_mutex_create_inner(RT_MUTEX *mutex, const char *name,
 
 int rt_mutex_create(RT_MUTEX *mutex, const char *name)
 {
-	xnarch_atomic_t *fastlock = NULL;
-	int err;
-
-#ifdef CONFIG_XENO_FASTSYNCH
-	/* Allocate lock memory for in-kernel use */
-	fastlock = xnmalloc(sizeof(xnarch_atomic_t));
-
-	if (!fastlock)
-		return -ENOMEM;
-#endif /* CONFIG_XENO_FASTSYNCH */
-
-	err = rt_mutex_create_inner(mutex, name, fastlock);
-
-#ifdef CONFIG_XENO_FASTSYNCH
-	if (err)
-		xnfree(fastlock);
-#endif /* CONFIG_XENO_FASTSYNCH */
-
-	return err;
-}
-
-int rt_mutex_delete_inner(RT_MUTEX *mutex)
-{
-	int err = 0, rc;
-	spl_t s;
-
-	if (xnpod_asynch_p())
-		return -EPERM;
-
-	xnlock_get_irqsave(&nklock, s);
-
-	mutex = xeno_h2obj_validate(mutex, XENO_MUTEX_MAGIC, RT_MUTEX);
-
-	if (!mutex) {
-		err = xeno_handle_error(mutex, XENO_MUTEX_MAGIC, RT_MUTEX);
-		goto unlock_and_exit;
-	}
-
-	removeq(mutex->rqueue, &mutex->rlink);
-
-	rc = xnsynch_destroy(&mutex->synch_base);
-
-	if (mutex->handle)
-		xnregistry_remove(mutex->handle);
-
-	xeno_mark_deleted(mutex);
-
-	if (rc == XNSYNCH_RESCHED)
-		/* Some task has been woken up as a result of the deletion:
-		   reschedule now. */
-		xnpod_schedule();
-
-      unlock_and_exit:
-
-	xnlock_put_irqrestore(&nklock, s);
-
-	return err;
+	return rt_mutex_create_inner(mutex, name, 1);
 }
 
 /**
@@ -312,22 +305,45 @@ int rt_mutex_delete_inner(RT_MUTEX *mutex)
 
 int rt_mutex_delete(RT_MUTEX *mutex)
 {
-	int err;
+	int err = 0, global = 0, rc;
+	spl_t s;
 
-	err = rt_mutex_delete_inner(mutex);
+	if (xnpod_asynch_p())
+		return -EPERM;
+
+	xnlock_get_irqsave(&nklock, s);
+
+	mutex = xeno_h2obj_validate(mutex, XENO_MUTEX_MAGIC, RT_MUTEX);
+
+	if (!mutex) {
+		err = xeno_handle_error(mutex, XENO_MUTEX_MAGIC, RT_MUTEX);
+		goto unlock_and_exit;
+	}
+
+	global = xnsynch_test_flags(&mutex->synch_base, RT_MUTEX_EXPORTED);
+
+	removeq(mutex->rqueue, &mutex->rlink);
+
+	rc = xnsynch_destroy(&mutex->synch_base);
+
+	if (mutex->handle)
+		xnregistry_remove(mutex->handle);
+
+	xeno_mark_deleted(mutex);
+
+	if (rc == XNSYNCH_RESCHED)
+		/* Some task has been woken up as a result of the deletion:
+		   reschedule now. */
+		xnpod_schedule();
+
+      unlock_and_exit:
+
+	xnlock_put_irqrestore(&nklock, s);
 
 #ifdef CONFIG_XENO_FASTSYNCH
-	if (!err) {
-#ifdef CONFIG_XENO_OPT_PERVASIVE
-		if (mutex->cpid) {
-			int global = xnsynch_test_flags(&mutex->synch_base,
-							RT_MUTEX_EXPORTED);
-			xnheap_free(&xnsys_ppd_get(global)->sem_heap,
-				    mutex->synch_base.fastlock);
-		} else
-#endif /* CONFIG_XENO_OPT_PERVASIVE */
-			xnfree(mutex->synch_base.fastlock);
-	}
+	if (!err)
+		xnheap_free(&xnsys_ppd_get(global)->sem_heap,
+			    mutex->synch_base.fastlock);
 #endif /* CONFIG_XENO_FASTSYNCH */
 
 	return err;
@@ -747,9 +763,9 @@ void __native_mutex_pkg_cleanup(void)
 
 /*@}*/
 
-EXPORT_SYMBOL(rt_mutex_create);
-EXPORT_SYMBOL(rt_mutex_delete);
-EXPORT_SYMBOL(rt_mutex_acquire);
-EXPORT_SYMBOL(rt_mutex_acquire_until);
-EXPORT_SYMBOL(rt_mutex_release);
-EXPORT_SYMBOL(rt_mutex_inquire);
+EXPORT_SYMBOL_GPL(rt_mutex_create);
+EXPORT_SYMBOL_GPL(rt_mutex_delete);
+EXPORT_SYMBOL_GPL(rt_mutex_acquire);
+EXPORT_SYMBOL_GPL(rt_mutex_acquire_until);
+EXPORT_SYMBOL_GPL(rt_mutex_release);
+EXPORT_SYMBOL_GPL(rt_mutex_inquire);
