@@ -2,7 +2,7 @@
  * @file
  * This file is part of the Xenomai project.
  *
- * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org> 
+ * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -126,7 +126,7 @@ static RT_TASK *__rt_task_current(struct task_struct *p)
  * a3: int prio;
  * a4: int mode;
  * a5: pthread_t opaque;
- * a6: thread mode writeback area;
+ * a6: thread mode offset writeback area;
  * }
  */
 
@@ -184,8 +184,10 @@ static int __rt_task_create(struct pt_regs *regs)
 	   the platform does not support it. */
 
 	err = rt_task_create(task, name, 0, prio, XNFPU | XNSHADOW | mode);
-	if (err)
+	if (err) {
+		task = NULL;
 		goto fail;
+	}
 
 	/* Apply CPU affinity */
 	set_cpus_allowed(p, task->affinity);
@@ -372,8 +374,8 @@ static int __rt_task_yield(struct pt_regs *regs)
 
 /*
  * int __rt_task_set_periodic(RT_TASK_PLACEHOLDER *ph,
- *			         RTIME idate,
- *			         RTIME period)
+ *				 RTIME idate,
+ *				 RTIME period)
  */
 
 static int __rt_task_set_periodic(struct pt_regs *regs)
@@ -589,19 +591,22 @@ static int __rt_task_set_mode(struct pt_regs *regs)
 	int err, setmask, clrmask, mode_r;
 
 	clrmask = __xn_reg_arg1(regs);
-	setmask = __xn_reg_arg2(regs);
-
-	err =
-	    rt_task_set_mode(clrmask & ~T_PRIMARY, setmask & ~T_PRIMARY,
-			     &mode_r);
+	if (clrmask & T_CONFORMING)
+		return -EINVAL;
 
+	/*
+	 * This call already required a primary mode switch, so if
+	 * T_CONFORMING was specified for a real-time shadow, we are
+	 * fine. If it was given from a non real-time shadow, well
+	 * this is silly, and we'll be relaxed soon due to the
+	 * auto-relax feature, leading to a nop.
+	 */
+	setmask = __xn_reg_arg2(regs) & ~T_CONFORMING;
+	err = rt_task_set_mode(clrmask, setmask, &mode_r);
 	if (err)
 		return err;
 
-	if ((clrmask & T_PRIMARY) != 0)
-		xnshadow_relax(0, 0);
-	else
-		mode_r |= T_PRIMARY;
+	mode_r |= T_CONFORMING;
 
 	if (__xn_reg_arg3(regs) &&
 	    __xn_safe_copy_to_user((void __user *)__xn_reg_arg3(regs),
@@ -1508,7 +1513,6 @@ static int __rt_event_inquire(struct pt_regs *regs)
 static int __rt_mutex_create(struct pt_regs *regs)
 {
 	char name[XNOBJECT_NAME_LEN];
-	xnarch_atomic_t *fastlock = NULL;
 	xnheap_t *sem_heap;
 	RT_MUTEX_PLACEHOLDER ph;
 	RT_MUTEX *mutex;
@@ -1531,39 +1535,31 @@ static int __rt_mutex_create(struct pt_regs *regs)
 	if (!mutex)
 		return -ENOMEM;
 
-#ifdef CONFIG_XENO_FASTSYNCH
-	fastlock = xnheap_alloc(sem_heap, sizeof(xnarch_atomic_t));
+	err = rt_mutex_create_inner(mutex, name, *name != '\0');
+	if (err < 0)
+		goto err_free_mutex;
 
-	if (!fastlock) {
-		xnfree(mutex);
-		return -ENOMEM;
-	}
-#endif /* CONFIG_XENO_FASTSYNCH */
-
-	err = rt_mutex_create_inner(mutex, name, fastlock);
-
-	if (err == 0) {
-		mutex->cpid = current->pid;
-		/* Copy back the registry handle to the ph struct. */
-		ph.opaque = mutex->handle;
+	mutex->cpid = current->pid;
+	/* Copy back the registry handle to the ph struct. */
+	ph.opaque = mutex->handle;
 #ifdef CONFIG_XENO_FASTSYNCH
 		/* The lock address will be finished in user space. */
-		ph.fastlock =
-			(void *)xnheap_mapped_offset(sem_heap, fastlock);
-		if (*name != '\0')
-			xnsynch_set_flags(&mutex->synch_base,
-					  RT_MUTEX_EXPORTED);
+	ph.fastlock =
+		(void *)xnheap_mapped_offset(sem_heap,
+					     mutex->synch_base.fastlock);
 #endif /* CONFIG_XENO_FASTSYNCH */
-		if (__xn_safe_copy_to_user((void __user *)__xn_reg_arg1(regs), &ph,
-					   sizeof(ph)))
-			err = -EFAULT;
-	} else {
-#ifdef CONFIG_XENO_FASTSYNCH
-		xnheap_free(&xnsys_ppd_get(*name != '\0')->sem_heap, fastlock);
-#endif /* CONFIG_XENO_FASTSYNCH */
-		xnfree(mutex);
+	if (__xn_safe_copy_to_user((void __user *)__xn_reg_arg1(regs), &ph,
+				   sizeof(ph))) {
+		err = -EFAULT;
+		goto err_delete_mutex;
 	}
 
+	return 0;
+
+  err_delete_mutex:
+	rt_mutex_delete(mutex);
+  err_free_mutex:
+	xnfree(mutex);
 	return err;
 }
 
@@ -1618,17 +1614,10 @@ static int __rt_mutex_delete(struct pt_regs *regs)
 	if (!mutex)
 		return -ESRCH;
 
-	err = rt_mutex_delete_inner(mutex);
+	err = rt_mutex_delete(mutex);
 
-	if (!err && mutex->cpid) {
-#ifdef CONFIG_XENO_FASTSYNCH
-		int global = xnsynch_test_flags(&mutex->synch_base,
-						RT_MUTEX_EXPORTED);
-		xnheap_free(&xnsys_ppd_get(global)->sem_heap,
-			    mutex->synch_base.fastlock);
-#endif /* CONFIG_XENO_FASTSYNCH */
+	if (!err && mutex->cpid)
 		xnfree(mutex);
-	}
 
 	return err;
 }
@@ -1829,18 +1818,24 @@ static int __rt_cond_delete(struct pt_regs *regs)
 
 /*
  * int __rt_cond_wait_prologue(RT_COND_PLACEHOLDER *cph,
- *                    	       RT_MUTEX_PLACEHOLDER *mph,
+ *		      	       RT_MUTEX_PLACEHOLDER *mph,
  *		      	       unsigned *plockcnt,
- *                    	       xntmode_t timeout_mode,
- *                    	       RTIME *timeoutp)
+ *		      	       xntmode_t timeout_mode,
+ *		      	       RTIME *timeoutp)
  */
 
+struct us_cond_data {
+	unsigned lockcnt;
+	int err;
+};
+
 static int __rt_cond_wait_prologue(struct pt_regs *regs)
 {
 	RT_COND_PLACEHOLDER cph, mph;
+	unsigned dummy, *plockcnt;
 	xntmode_t timeout_mode;
+	struct us_cond_data d;
 	int err, perr = 0;
-	unsigned lockcnt;
 	RT_MUTEX *mutex;
 	RT_COND *cond;
 	RTIME timeout;
@@ -1869,26 +1864,35 @@ static int __rt_cond_wait_prologue(struct pt_regs *regs)
 				     sizeof(timeout)))
 		return -EFAULT;
 
-	err = rt_cond_wait_prologue(cond, mutex, &lockcnt, timeout_mode, timeout);
+#ifdef CONFIG_XENO_FASTSYNCH
+	if (__xn_safe_copy_from_user(&d, (void __user *)__xn_reg_arg3(regs),
+				     sizeof(d)))
+		return -EFAULT;
+
+	plockcnt = &dummy;
+#else /* !CONFIG_XENO_FASTSYNCH */
+	plockcnt = &d.lockcnt;
+	(void)dummy;
+#endif /* !CONFIG_XENO_FASTSYNCH */
+
+	err = rt_cond_wait_prologue(cond, mutex, plockcnt, timeout_mode, timeout);
 
 	switch(err) {
 	case 0:
 	case -ETIMEDOUT:
 	case -EIDRM:
-		perr = rt_task_errno = err;
-		err = rt_cond_wait_epilogue(mutex, lockcnt);
+		perr = d.err = err;
+		err = rt_cond_wait_epilogue(mutex, *plockcnt);
 		break;
 
 	case -EINTR:
 		perr = err;
-		rt_task_errno = 0; /* epilogue should return 0. */
+		d.err = 0; /* epilogue should return 0. */
 		break;
 	}
 
-	if (err == -EINTR
-	    && __xn_reg_arg3(regs)
-	    && __xn_safe_copy_to_user((void __user *)__xn_reg_arg3(regs),
-				      &lockcnt, sizeof(lockcnt)))
+	if (__xn_safe_copy_to_user((void __user *)__xn_reg_arg3(regs),
+				   &d, sizeof(d)))
 		return -EFAULT;
 
 	return err == 0 ? perr : err;
@@ -1918,7 +1922,7 @@ static int __rt_cond_wait_epilogue(struct pt_regs *regs)
 
 	err = rt_cond_wait_epilogue(mutex, lockcnt);
 
-	return err == 0 ? rt_task_errno : err;
+	return err;
 }
 
 /*
@@ -1999,7 +2003,7 @@ static int __rt_cond_inquire(struct pt_regs *regs)
 #else /* !CONFIG_XENO_OPT_NATIVE_COND */
 
 #define __rt_cond_create    	__rt_call_not_available
-#define __rt_cond_bind      	__rt_call_not_available
+#define __rt_cond_bind	    	__rt_call_not_available
 #define __rt_cond_delete    	__rt_call_not_available
 #define __rt_cond_wait_prologue __rt_call_not_available
 #define __rt_cond_wait_epilogue __rt_call_not_available
@@ -2060,7 +2064,7 @@ static int __rt_queue_create(struct pt_regs *regs)
 	ph.opaque = q->handle;
 	ph.opaque2 = &q->bufpool;
 	ph.mapsize = xnheap_extentsize(&q->bufpool);
-	xnheap_area_set(&ph, xnheap_base_memory(&q->bufpool));
+	ph.area = xnheap_base_memory(&q->bufpool);
 	if (__xn_safe_copy_to_user((void __user *)__xn_reg_arg1(regs), &ph, sizeof(ph)))
 		return -EFAULT;
 
@@ -2098,7 +2102,7 @@ static int __rt_queue_bind(struct pt_regs *regs)
 
 	ph.opaque2 = &q->bufpool;
 	ph.mapsize = xnheap_extentsize(&q->bufpool);
-	xnheap_area_set(&ph, xnheap_base_memory(&q->bufpool));
+	ph.area = xnheap_base_memory(&q->bufpool);
 	xnlock_put_irqrestore(&nklock, s);
 
 	if (__xn_safe_copy_to_user((void __user *)__xn_reg_arg1(regs), &ph, sizeof(ph)))
@@ -2374,9 +2378,9 @@ static int __rt_queue_receive(struct pt_regs *regs)
 	if (__xn_safe_copy_from_user(&timeout, (void __user *)__xn_reg_arg4(regs),
 				     sizeof(timeout)))
 		return -EFAULT;
-	
+
 	timeout_mode = __xn_reg_arg3(regs);
-	
+
 	xnlock_get_irqsave(&nklock, s);
 
 	q = (RT_QUEUE *)xnregistry_fetch(ph.opaque);
@@ -2447,7 +2451,7 @@ static int __rt_queue_read(struct pt_regs *regs)
 
 	/* Relative/absolute timeout spec. */
 	timeout_mode = __xn_reg_arg4(regs);
-	
+
 	if (__xn_safe_copy_from_user(&timeout, (void __user *)__xn_reg_arg5(regs),
 				     sizeof(timeout)))
 		return -EFAULT;
@@ -2584,7 +2588,7 @@ static int __rt_heap_create(struct pt_regs *regs)
 	ph.opaque = heap->handle;
 	ph.opaque2 = &heap->heap_base;
 	ph.mapsize = xnheap_extentsize(&heap->heap_base);
-	xnheap_area_set(&ph, xnheap_base_memory(&heap->heap_base));
+	ph.area = xnheap_base_memory(&heap->heap_base);
 	if (__xn_safe_copy_to_user((void __user *)__xn_reg_arg1(regs), &ph, sizeof(ph)))
 		return -EFAULT;
 
@@ -2622,7 +2626,7 @@ static int __rt_heap_bind(struct pt_regs *regs)
 
 	ph.opaque2 = &heap->heap_base;
 	ph.mapsize = xnheap_extentsize(&heap->heap_base);
-	xnheap_area_set(&ph, xnheap_base_memory(&heap->heap_base));
+	ph.area = xnheap_base_memory(&heap->heap_base);
 
 	xnlock_put_irqrestore(&nklock, s);
 
@@ -2820,7 +2824,7 @@ void rt_alarm_handler(RT_ALARM *alarm, void *cookie)
 	xnsynch_flush(&alarm->synch_base, 0);
 }
 
-EXPORT_SYMBOL(rt_alarm_handler);
+EXPORT_SYMBOL_GPL(rt_alarm_handler);
 
 /*
  * int __rt_alarm_create(RT_ALARM_PLACEHOLDER *ph,
@@ -3054,7 +3058,7 @@ int rt_intr_handler(xnintr_t *cookie)
 	return XN_ISR_HANDLED | (intr->mode & XN_ISR_NOENABLE);
 }
 
-EXPORT_SYMBOL(rt_intr_handler);
+EXPORT_SYMBOL_GPL(rt_intr_handler);
 
 /*
  * int __rt_intr_create(RT_INTR_PLACEHOLDER *ph,
@@ -4107,10 +4111,10 @@ static xnsysent_t __systab[] = {
 	[__native_cond_create] = {&__rt_cond_create, __xn_exec_any},
 	[__native_cond_bind] = {&__rt_cond_bind, __xn_exec_conforming},
 	[__native_cond_delete] = {&__rt_cond_delete, __xn_exec_any},
-	[__native_cond_wait_prologue] = 
-		{&__rt_cond_wait_prologue, 
+	[__native_cond_wait_prologue] =
+		{&__rt_cond_wait_prologue,
 		 __xn_exec_primary | __xn_exec_norestart},
-	[__native_cond_wait_epilogue] = 
+	[__native_cond_wait_epilogue] =
 		{&__rt_cond_wait_epilogue, __xn_exec_primary},
 	[__native_cond_signal] = {&__rt_cond_signal, __xn_exec_any},
 	[__native_cond_broadcast] = {&__rt_cond_broadcast, __xn_exec_any},
@@ -4171,7 +4175,6 @@ static struct xnskin_props __props = {
 	.nrcalls = sizeof(__systab) / sizeof(__systab[0]),
 	.systab = __systab,
 	.eventcb = &__shadow_eventcb,
-	.sig_unqueue = NULL,
 	.timebasep = &__native_tbase,
 	.module = THIS_MODULE
 };
