@@ -222,9 +222,6 @@ void pse51_mutex_destroy_internal(pse51_mutex_t *mutex,
 	xnheap_free(&xnsys_ppd_get(mutex->attr.pshared)->sem_heap,
 		    mutex->synchbase.fastlock);
 #endif /* CONFIG_XENO_FASTSYNCH */
-	/* We do not free the owner if the mutex is not pshared, because when
-	   this function is called from pse51_mutexq_cleanup, the sem_heap has
-	   been destroyed, and we have no way to find it back. */
 	xnfree(mutex);
 }
 
@@ -668,9 +665,9 @@ void pse51_mutex_pkg_cleanup(void)
 
 /*@}*/
 
-EXPORT_SYMBOL(pthread_mutex_init);
-EXPORT_SYMBOL(pthread_mutex_destroy);
-EXPORT_SYMBOL(pthread_mutex_trylock);
-EXPORT_SYMBOL(pthread_mutex_lock);
-EXPORT_SYMBOL(pthread_mutex_timedlock);
-EXPORT_SYMBOL(pthread_mutex_unlock);
+EXPORT_SYMBOL_GPL(pthread_mutex_init);
+EXPORT_SYMBOL_GPL(pthread_mutex_destroy);
+EXPORT_SYMBOL_GPL(pthread_mutex_trylock);
+EXPORT_SYMBOL_GPL(pthread_mutex_lock);
+EXPORT_SYMBOL_GPL(pthread_mutex_timedlock);
+EXPORT_SYMBOL_GPL(pthread_mutex_unlock);
