@@ -43,7 +43,7 @@
  * The action to be taken upon reception of a signal depends on the thread
  * signal mask, (see pthread_sigmask()), and on the settings described by a
  * @b sigaction structure (see sigaction()).
- * 
+ *
  *@{*/
 
 #ifdef CONFIG_XENO_OPT_PERVASIVE
@@ -168,7 +168,7 @@ static inline void andnotset(pse51_sigset_t *set,
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigemptyset.html">
  * Specification.</a>
- * 
+ *
  */
 int sigemptyset(sigset_t * set)
 {
@@ -191,7 +191,7 @@ int sigemptyset(sigset_t * set)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigfillset.html">
  * Specification.</a>
- * 
+ *
  */
 int sigfillset(sigset_t * set)
 {
@@ -219,7 +219,7 @@ int sigfillset(sigset_t * set)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigaddset.html">
  * Specification.</a>
- * 
+ *
  */
 int sigaddset(sigset_t * set, int sig)
 {
@@ -252,7 +252,7 @@ int sigaddset(sigset_t * set, int sig)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigdelset.html">
  * Specification.</a>
- * 
+ *
  */
 int sigdelset(sigset_t * set, int sig)
 {
@@ -285,7 +285,7 @@ int sigdelset(sigset_t * set, int sig)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigismember.html">
  * Specification.</a>
- * 
+ *
  */
 int sigismember(const sigset_t * set, int sig)
 {
@@ -467,7 +467,7 @@ static pse51_siginfo_t *pse51_getsigq(pse51_sigqueue_t * queue,
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigaction.html">
  * Specification.</a>
- * 
+ *
  */
 int sigaction(int sig, const struct sigaction *act, struct sigaction *oact)
 {
@@ -522,36 +522,75 @@ int sigaction(int sig, const struct sigaction *act, struct sigaction *oact)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/pthread_kill.html">
  * Specification.</a>
- * 
+ *
  */
 int pthread_kill(pthread_t thread, int sig)
 {
 	pse51_siginfo_t *si = NULL;
+	int ret = 0;
 	spl_t s;
 
-	if ((unsigned)sig > SIGRTMAX)
-		return EINVAL;
-
-	if (sig) {
-		si = pse51_new_siginfo(sig, SI_USER, (union sigval)0);
-
-		if (!si)
-			return EAGAIN;
-	}
-
 	xnlock_get_irqsave(&nklock, s);
 
 	if (!pse51_obj_active(thread, PSE51_THREAD_MAGIC, struct pse51_thread)) {
-		xnlock_put_irqrestore(&nklock, s);
-		return ESRCH;
+		ret = ESRCH;
+		goto unlock_and_exit;
 	}
 
-	if (sig && pse51_sigqueue_inner(thread, si))
-		xnpod_schedule();
+	if (sig == 0)
+		/* Test for existence -- take fast path. */
+		goto unlock_and_exit;
+
+	/*
+	 * Undocumented pseudo-signals to suspend/resume/unblock
+	 * threads via the low-level nucleus services. Process them
+	 * early, before anyone can notice...
+	 */
+	if (sig == SIGSUSP) {
+		/*
+		 * The self-suspension case for shadows was handled at
+		 * call site: we must be in primary mode already.
+		 */
+		xnpod_suspend_thread(&thread->threadbase, XNSUSP,
+				     XN_INFINITE, XN_RELATIVE, NULL);
+		if (&thread->threadbase == xnpod_current_thread() &&
+		    xnthread_test_info(&thread->threadbase, XNBREAK))
+			ret = EINTR;
+		goto unlock_and_exit;
+	}
+
+	if (sig == SIGRESM) {
+		xnpod_resume_thread(&thread->threadbase, XNSUSP);
+		goto resched;
+	}
+
+	if (sig == SIGRELS) {
+		xnpod_unblock_thread(&thread->threadbase);
+		goto resched;
+	}
+
+	if ((unsigned)sig > SIGRTMAX) {
+		ret = EINVAL;
+		goto unlock_and_exit;
+	}
+
+	si = pse51_new_siginfo(sig, SI_USER, (union sigval)0);
+	if (si == NULL) {
+		ret = EAGAIN;
+		goto unlock_and_exit;
+	}
+
+	if (!pse51_sigqueue_inner(thread, si))
+		goto unlock_and_exit;
+
+resched:
+	xnpod_schedule();
+
+unlock_and_exit:
 
 	xnlock_put_irqrestore(&nklock, s);
 
-	return 0;
+	return ret;
 }
 
 /**
@@ -580,7 +619,7 @@ int pthread_kill(pthread_t thread, int sig)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigqueue.html">
  * sigqueue() specification.</a>
- * 
+ *
  */
 int pthread_sigqueue_np(pthread_t thread, int sig, union sigval value)
 {
@@ -631,7 +670,7 @@ int pthread_sigqueue_np(pthread_t thread, int sig, union sigval value)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigpending.html">
  * Specification.</a>
- * 
+ *
  */
 int sigpending(sigset_t * set)
 {
@@ -694,7 +733,7 @@ int sigpending(sigset_t * set)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/pthread_sigmask.html">
  * Specification.</a>
- * 
+ *
  */
 int pthread_sigmask(int how, const sigset_t * set, sigset_t * oset)
 {
@@ -867,7 +906,7 @@ static int pse51_sigtimedwait_inner(const sigset_t * set,
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigwait.html">
  * Specification.</a>
- * 
+ *
  */
 int sigwait(const sigset_t * set, int *sig)
 {
@@ -908,7 +947,7 @@ int sigwait(const sigset_t * set, int *sig)
 * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigwaitinfo.html">
  * Specification.</a>
- * 
+ *
  */
 int sigwaitinfo(const sigset_t * __restrict__ set,
 		siginfo_t * __restrict__ info)
@@ -961,7 +1000,7 @@ int sigwaitinfo(const sigset_t * __restrict__ set,
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/sigtimedwait.html">
  * Specification.</a>
- * 
+ *
  */
 int sigtimedwait(const sigset_t * __restrict__ set,
 		 siginfo_t * __restrict__ info,
@@ -1055,9 +1094,11 @@ static void pse51_dispatch_signals(xnsigmask_t sigs)
 #ifdef CONFIG_XENO_OPT_PERVASIVE
 static void pse51_dispatch_shadow_signals(xnsigmask_t sigs)
 {
+	spl_t dummy;
 	/* Migrate to secondary mode in order to get the signals delivered by
 	   Linux. */
 	xnshadow_relax(1, SIGDEBUG_MIGRATE_SIGNAL);
+	xnlock_get_irqsave(&nklock, dummy);
 }
 
 void pse51_signal_handle_request(pthread_t thread)
@@ -1185,17 +1226,17 @@ static void pse51_default_handler(int sig)
 
 /*@}*/
 
-EXPORT_SYMBOL(sigemptyset);
-EXPORT_SYMBOL(sigfillset);
-EXPORT_SYMBOL(sigaddset);
-EXPORT_SYMBOL(sigdelset);
-EXPORT_SYMBOL(sigismember);
-EXPORT_SYMBOL(pthread_kill);
-EXPORT_SYMBOL(pthread_sigmask);
-EXPORT_SYMBOL(pthread_sigqueue_np);
-EXPORT_SYMBOL(pse51_sigaction);
+EXPORT_SYMBOL_GPL(sigemptyset);
+EXPORT_SYMBOL_GPL(sigfillset);
+EXPORT_SYMBOL_GPL(sigaddset);
+EXPORT_SYMBOL_GPL(sigdelset);
+EXPORT_SYMBOL_GPL(sigismember);
+EXPORT_SYMBOL_GPL(pthread_kill);
+EXPORT_SYMBOL_GPL(pthread_sigmask);
+EXPORT_SYMBOL_GPL(pthread_sigqueue_np);
+EXPORT_SYMBOL_GPL(pse51_sigaction);
 
-EXPORT_SYMBOL(sigpending);
-EXPORT_SYMBOL(sigwait);
-EXPORT_SYMBOL(sigwaitinfo);
-EXPORT_SYMBOL(sigtimedwait);
+EXPORT_SYMBOL_GPL(sigpending);
+EXPORT_SYMBOL_GPL(sigwait);
+EXPORT_SYMBOL_GPL(sigwaitinfo);
+EXPORT_SYMBOL_GPL(sigtimedwait);
