@@ -25,7 +25,7 @@
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/xsh_chap02_09.html#tag_02_09">
  * Specification.</a>
- * 
+ *
  *@{*/
 
 #include <posix/thread.h>
@@ -297,7 +297,7 @@ int pthread_create(pthread_t *tid,
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/pthread_detach.html">
  * Specification.</a>
- * 
+ *
  */
 int pthread_detach(pthread_t thread)
 {
@@ -339,12 +339,12 @@ int pthread_detach(pthread_t thread)
  * @param t2 other thread identifier.
  *
  * @return a non zero value if the thread identifiers are equal;
- * @return 0 otherwise. 
+ * @return 0 otherwise.
  *
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/pthread_equal.html">
  * Specification.</a>
- * 
+ *
  */
 int pthread_equal(pthread_t t1, pthread_t t2)
 {
@@ -367,7 +367,7 @@ int pthread_equal(pthread_t t1, pthread_t t2)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/pthread_exit.html">
  * Specification.</a>
- * 
+ *
  */
 void pthread_exit(void *value_ptr)
 {
@@ -425,7 +425,7 @@ void pthread_exit(void *value_ptr)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/pthread_join.html">
  * Specification.</a>
- * 
+ *
  */
 int pthread_join(pthread_t thread, void **value_ptr)
 {
@@ -500,7 +500,7 @@ int pthread_join(pthread_t thread, void **value_ptr)
 				xnlock_put_irqrestore(&nklock, s);
 				return EINVAL;
 			}
-			
+
 			is_last_joiner = (--thread->nrt_joiners == 0);
 		}
 #endif /* __KERNEL__ */
@@ -534,7 +534,7 @@ int pthread_join(pthread_t thread, void **value_ptr)
  * @see
  * <a href="http://www.opengroup.org/onlinepubs/000095399/functions/pthread_self.html">
  * Specification.</a>
- * 
+ *
  */
 pthread_t pthread_self(void)
 {
@@ -661,7 +661,7 @@ int pthread_wait_np(unsigned long *overruns_r)
  * @return 0 on success;
  * @return an error number if:
  * - EINVAL, some bit in @a clrmask or @a setmask is invalid.
- * 
+ *
  */
 int pthread_set_mode_np(int clrmask, int setmask)
 {
@@ -682,6 +682,10 @@ int pthread_set_mode_np(int clrmask, int setmask)
 			      clrmask & ~XNTHREAD_STATE_SPARE1,
 			      setmask & ~XNTHREAD_STATE_SPARE1);
 
+	if ((clrmask & ~setmask) & XNLOCK)
+		/* Reschedule if the scheduler has been unlocked. */
+		xnpod_schedule();
+
 #ifdef CONFIG_XENO_OPT_PERVASIVE
 	if (xnthread_test_state(cur, XNSHADOW) && (clrmask & XNTHREAD_STATE_SPARE1) != 0)
 		xnshadow_relax(0, 0);
@@ -705,7 +709,7 @@ int pthread_set_mode_np(int clrmask, int setmask)
  * @return 0 on success;
  * @return an error number if:
  * - ESRCH, @a thread is invalid.
- * 
+ *
  */
 int pthread_set_name_np(pthread_t thread, const char *name)
 {
@@ -782,13 +786,13 @@ void pse51_thread_pkg_cleanup(void)
 
 /*@}*/
 
-EXPORT_SYMBOL(pthread_create);
-EXPORT_SYMBOL(pthread_detach);
-EXPORT_SYMBOL(pthread_equal);
-EXPORT_SYMBOL(pthread_exit);
-EXPORT_SYMBOL(pthread_join);
-EXPORT_SYMBOL(pthread_self);
-EXPORT_SYMBOL(pthread_make_periodic_np);
-EXPORT_SYMBOL(pthread_wait_np);
-EXPORT_SYMBOL(pthread_set_name_np);
-EXPORT_SYMBOL(pthread_set_mode_np);
+EXPORT_SYMBOL_GPL(pthread_create);
+EXPORT_SYMBOL_GPL(pthread_detach);
+EXPORT_SYMBOL_GPL(pthread_equal);
+EXPORT_SYMBOL_GPL(pthread_exit);
+EXPORT_SYMBOL_GPL(pthread_join);
+EXPORT_SYMBOL_GPL(pthread_self);
+EXPORT_SYMBOL_GPL(pthread_make_periodic_np);
+EXPORT_SYMBOL_GPL(pthread_wait_np);
+EXPORT_SYMBOL_GPL(pthread_set_name_np);
+EXPORT_SYMBOL_GPL(pthread_set_mode_np);
