@@ -25,7 +25,7 @@
 #include <native/cond.h>
 #endif /* __NATIVE_SKIN */
 #include <asm-generic/xenomai/bits/current.h> /* For internal use, do not use
-				         in your code. */
+					 in your code. */
 #include <asm-generic/xenomai/stack.h>
 
 #define MUTEX_CREATE		1
@@ -40,6 +40,7 @@
 #define COND_DESTROY		10
 #define THREAD_DETACH		11
 #define THREAD_CREATE		12
+#define THREAD_JOIN		13
 
 #define NS_PER_MS	1000000
 
@@ -84,7 +85,7 @@ void check_current_prio(int expected_prio)
 # ifdef __pse51_get_current_prio
 	extern unsigned __pse51_muxid;
 
-        XENOMAI_SKINCALL1(__pse51_muxid, __pse51_get_current_prio, &current_prio);
+	XENOMAI_SKINCALL1(__pse51_muxid, __pse51_get_current_prio, &current_prio);
 # else /* !__pse51_get_current_prio */
 	current_prio = expected_prio;
 # endif /* !__pse51_get_current_prio */
@@ -113,11 +114,11 @@ void check_current_mode(int expected_primary_mode)
 {
 	int current_in_primary;
 
-        /* This is a unit test, and in this circonstance, we are allowed to
+	/* This is a unit test, and in this circonstance, we are allowed to
 	   call xeno_get_current_mode. But please do not do that in your
 	   own code. */
 	current_in_primary = !(xeno_get_current_mode() & XNRELAX);
-	
+
 	if (current_in_primary != expected_primary_mode) {
 		fprintf(stderr,
 			"FAILURE: current mode (%d) != expected mode (%d)\n",
@@ -135,7 +136,7 @@ void yield(void)
 #endif /* __NATIVE_SKIN__ */
 }
 
-int dispatch(const char *service_name, 
+int dispatch(const char *service_name,
 	     int service_type, int check, int expected, ...)
 {
 	unsigned long long timeout;
@@ -235,7 +236,7 @@ int dispatch(const char *service_name,
 		status = -rt_cond_signal(va_arg(ap, RT_COND *));
 #endif /* __NATIVE_SKIN__ */
 		break;
-	
+
 	case COND_WAIT:
 #ifdef XENO_POSIX
 		cond = va_arg(ap, pthread_cond_t *);
@@ -282,11 +283,21 @@ int dispatch(const char *service_name,
 		thread = va_arg(ap, RT_TASK *);
 		prio = va_arg(ap, int);
 		handler = va_arg(ap, void *);
-		status = -rt_task_spawn(thread, NULL, 0, prio, 0, handler,
+		status = -rt_task_spawn(thread, NULL, 0, prio, T_JOINABLE, handler,
 					va_arg(ap, void *));
 #endif /* __NATIVE_SKIN__ */
 		break;
 
+	case THREAD_JOIN:
+#ifdef XENO_POSIX
+		thread = va_arg(ap, pthread_t *);
+		status = pthread_join(*thread, NULL);
+#else /* __NATIVE_SKIN__ */
+		thread = va_arg(ap, RT_TASK *);
+		status = rt_task_join(thread);
+#endif /* __NATIVE_SKIN__ */
+		break;
+
 	default:
 		fprintf(stderr, "Unknown service %i.\n", service_type);
 		exit(EXIT_FAILURE);
@@ -294,7 +305,7 @@ int dispatch(const char *service_name,
 	va_end(ap);
 
 	if (check && status != expected) {
-		fprintf(stderr, "FAILURE: %s: %i (%s) instead of %i\n", 
+		fprintf(stderr, "FAILURE: %s: %i (%s) instead of %i\n",
 			service_name, status, strerror(status), expected);
 		exit(EXIT_FAILURE);
 	}
@@ -366,7 +377,7 @@ void recursive_wait(void)
 	dispatch("rec thread_create", THREAD_CREATE, 1, 0, &waiter_tid, 2,
 		 waiter, &mutex);
 
-	dispatch("rec mutex_unlock 2", MUTEX_UNLOCK, 1, 0, &mutex); 
+	dispatch("rec mutex_unlock 2", MUTEX_UNLOCK, 1, 0, &mutex);
 	ms_sleep(11);
 	dispatch("rec mutex_unlock 1", MUTEX_UNLOCK, 1, 0, &mutex);
 	yield();
@@ -668,8 +679,6 @@ void *cond_signaler(void *cookie)
 	struct cond_mutex *cm = (struct cond_mutex *) cookie;
 	unsigned long long start, diff;
 
-	dispatch("cond_signaler pthread_detach", THREAD_DETACH, 1, 0);
-
 	start = rt_timer_tsc();
 	dispatch("cond_signaler mutex_lock 1", MUTEX_LOCK, 1, 0, cm->mutex);
 	diff = rt_timer_tsc2ns(rt_timer_tsc() - start);
@@ -733,6 +742,8 @@ void simple_condwait(void)
 
 	dispatch("simple_condwait mutex_destroy", MUTEX_DESTROY, 1, 0, &mutex);
 	dispatch("simple_condwait cond_destroy", COND_DESTROY, 1, 0, &cond);
+
+	dispatch("simple_condwait join", THREAD_JOIN, 1, 0, &cond_signaler_tid);
 }
 
 void recursive_condwait(void)
@@ -772,6 +783,8 @@ void recursive_condwait(void)
 
 	dispatch("rec_condwait mutex_destroy", MUTEX_DESTROY, 1, 0, &mutex);
 	dispatch("rec_condwait cond_destroy", COND_DESTROY, 1, 0, &cond);
+
+	dispatch("rec_condwait join", THREAD_JOIN, 1, 0, &cond_signaler_tid);
 }
 
 int main(void)
@@ -806,4 +819,3 @@ int main(void)
 	fprintf(stderr, "Test OK\n");
 	return 0;
 }
-
